@@ -14,16 +14,24 @@ use MIME::Base64;
 use Safe;
 use Encode;
 
-our $VERSION = '2.0.14';
+our $VERSION = '2.22.0';
 
 my $dataStart = tell(DATA);
 
 ## @method public string serviceToXML
 # Return all SAML parameters in well formated XML format, corresponding to
 # SAML 2 description.
+# Supported options:
+# * portal: override the portal URL
+# * signing_keys: override the list of signing keys
+# * encryption_keys override the list of encryption keys
 # @return string
 sub serviceToXML {
-    my ( $self, $conf, $type ) = @_;
+    my ( $self, $conf, $type, %options ) = @_;
+
+    my $portal =
+      $options{portal} || $conf->{portal} || "http://auth.example.com/";
+    $portal =~ s/\/$//;
 
     seek DATA, $dataStart, 0;
     my $s = join '', <DATA>;
@@ -50,7 +58,7 @@ sub serviceToXML {
     }
 
     foreach (@param_auto) {
-        $template->param( $_, $self->getValue( $_, $conf ) );
+        $template->param( $_, $self->_replacePortal( $conf->{$_}, $portal ) );
     }
 
   # When asked to provide only IDP metadata, take into account EntityID override
@@ -66,69 +74,30 @@ sub serviceToXML {
     );
 
     foreach (@param_boolean) {
-        $template->param( $_, $self->getValue( $_, $conf ) ? 'true' : 'false' );
+        $template->param( $_,
+            $self->_replacePortal( $conf->{$_}, $portal ) ? 'true' : 'false' );
     }
 
-    # Format public keys
-    my @param_keys = qw(
-      samlServicePublicKeySig
-      samlServicePublicKeyEnc
+    my $signing_keys =
+      $options{signing_keys} || [ $conf->{samlServicePublicKeySig} ];
+    my $encryption_keys = $options{encryption_keys}
+      || (
+        $conf->{samlServicePublicKeyEnc}
+        ? [ $conf->{samlServicePublicKeyEnc} ]
+        : $signing_keys
+      );
+
+    # Keys
+    $template->param( "signing_keys",
+        [ map { { keyInfo => $self->_getKeyInfoFromPem($_) } } @$signing_keys ]
+    );
+    $template->param(
+        "encryption_keys",
+        [
+            map { { keyInfo => $self->_getKeyInfoFromPem($_) } }
+              @$encryption_keys
+        ]
     );
-
-    foreach (@param_keys) {
-        my $str = '';
-        my $val = $self->getValue( $_, $conf );
-
-        # A default value for samlServicePublicKeyEnc parameter
-        if ( $_ =~ /samlServicePublicKeyEnc/ ) {
-            unless ( $val && length $val gt 0 ) {
-                $val = $conf->{samlServicePublicKeySig};
-            }
-        }
-
-        # Generate XML
-        if ( defined $val && length $val gt 0 ) {
-
-            # Public Key ?
-            if ( $val =~ /^-----BEGIN PUBLIC KEY-----/
-                and my $rsa_pub = Crypt::OpenSSL::RSA->new_public_key($val) )
-            {
-                my @params = $rsa_pub->get_key_parameters();
-                my $mod    = encode_base64( $params[0]->to_bin() );
-                my $exp    = encode_base64( $params[1]->to_bin() );
-                $str =
-                    '<ds:KeyValue>' . "\n\t"
-                  . '<RSAKeyValue xmlns="http://www.w3.org/2000/09/xmldsig#">'
-                  . "\n\t\t"
-                  . '<Modulus>'
-                  . $mod
-                  . '</Modulus>'
-                  . "\n\t\t"
-                  . '<Exponent>'
-                  . $exp
-                  . '</Exponent>' . "\n\t"
-                  . '</RSAKeyValue>' . "\n"
-                  . '</ds:KeyValue>';
-            }
-
-            # Certificate ?
-            if ( $val =~ /^-----BEGIN CERTIFICATE-----/
-                and my $certificate =
-                Crypt::OpenSSL::X509->new_from_string($val) )
-            {
-                $certificate = $certificate->as_string();
-                $certificate =~ s/^-----BEGIN CERTIFICATE-----\n?//g;
-                $certificate =~ s/\n?-----END CERTIFICATE-----$//g;
-                $str =
-                    '<ds:X509Data>' . "\n\t"
-                  . '<ds:X509Certificate>' . "\n\t"
-                  . $certificate
-                  . '</ds:X509Certificate>' . "\n"
-                  . '</ds:X509Data>';
-            }
-        }
-        $template->param( $_, $str );
-    }
 
     # Rebuilded parameters for SAML services
     # A samlService value is formated like the following:
@@ -148,7 +117,7 @@ sub serviceToXML {
     );
 
     foreach (@param_service) {
-        my @_tab = split( /;/, $self->getValue( $_, $conf ) );
+        my @_tab = split( /;/, $self->_replacePortal( $conf->{$_}, $portal ) );
         $template->param( $_ . 'Binding',          $_tab[0] );
         $template->param( $_ . 'Location',         $_tab[1] );
         $template->param( $_ . 'ResponseLocation', $_tab[2] );
@@ -166,7 +135,7 @@ sub serviceToXML {
 
     my %indexed_endpoints;
     foreach (@param_assertion) {
-        my @_tab = split( /;/, $self->getValue( $_, $conf ) );
+        my @_tab = split( /;/, $self->_replacePortal( $conf->{$_}, $portal ) );
         $indexed_endpoints{ $_ . 'Default' }  = ( $_tab[0] ? 'true' : 'false' );
         $indexed_endpoints{ $_ . 'Index' }    = $_tab[1];
         $indexed_endpoints{ $_ . 'Binding' }  = $_tab[2];
@@ -186,7 +155,57 @@ sub serviceToXML {
     return $template->output;
 }
 
-#@method string getValue(string key, hashref conf)
+sub _getKeyInfoFromPem {
+    my ( $self, $val ) = @_;
+
+    my $str;
+
+    if ( defined $val and ref($val) eq "HASH" and exists $val->{public} ) {
+        $val = $val->{public};
+    }
+    if ( defined $val && length $val gt 0 ) {
+
+        # Public Key ?
+        if ( $val =~ /^-----BEGIN PUBLIC KEY-----/
+            and my $rsa_pub = Crypt::OpenSSL::RSA->new_public_key($val) )
+        {
+            my @params = $rsa_pub->get_key_parameters();
+            my $mod    = encode_base64( $params[0]->to_bin() );
+            my $exp    = encode_base64( $params[1]->to_bin() );
+            $str =
+                '<ds:KeyValue>' . "\n\t"
+              . '<RSAKeyValue xmlns="http://www.w3.org/2000/09/xmldsig#">'
+              . "\n\t\t"
+              . '<Modulus>'
+              . $mod
+              . '</Modulus>'
+              . "\n\t\t"
+              . '<Exponent>'
+              . $exp
+              . '</Exponent>' . "\n\t"
+              . '</RSAKeyValue>' . "\n"
+              . '</ds:KeyValue>';
+        }
+
+        # Certificate ?
+        if ( $val =~ /^-----BEGIN CERTIFICATE-----/
+            and my $certificate = Crypt::OpenSSL::X509->new_from_string($val) )
+        {
+            $certificate = $certificate->as_string();
+            $certificate =~ s/^-----BEGIN CERTIFICATE-----\n?//g;
+            $certificate =~ s/\n?-----END CERTIFICATE-----$//g;
+            $str =
+                '<ds:X509Data>' . "\n\t"
+              . '<ds:X509Certificate>' . "\n\t"
+              . $certificate
+              . '</ds:X509Certificate>' . "\n"
+              . '</ds:X509Data>';
+        }
+    }
+    return $str;
+}
+
+# DEPRECATED, remove in 3.0
 # Get the value for a metadata configuration key
 # Replace #PORTAL# macro
 # @param key Configuration key
@@ -199,9 +218,18 @@ sub getValue {
     my $portal = $conf->{portal} || "http://auth.example.com/";
     $portal =~ s/\/$//;
 
-    # Try to get value for the given key in configuraiton
-    my $value = $conf->{$key};
-    return unless defined $value;
+    return $self->_replacePortal( $conf->{$key}, $portal );
+}
+
+#@method string replacePortal(string key, hashref conf)
+# Get the value for a metadata configuration key
+# Replace #PORTAL# macro
+# @param key Configuration key
+# @param conf Configuration hash ref
+# @return value
+sub _replacePortal {
+    my ( $self, $value, $portal ) = @_;
+    return unless $value;
 
     # Replace #PORTAL# macro
     $value =~ s/#PORTAL#/$portal/g;
@@ -222,36 +250,38 @@ __DATA__
   <IDPSSODescriptor
       WantAuthnRequestsSigned="<TMPL_VAR NAME="samlIDPSSODescriptorWantAuthnRequestsSigned">"
       protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
-    <KeyDescriptor use="signing">
+    <TMPL_LOOP name="signing_keys"><KeyDescriptor use="signing">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
-          <TMPL_VAR NAME="samlServicePublicKeySig">
+          <TMPL_VAR NAME="keyInfo">
       </ds:KeyInfo>
-    </KeyDescriptor>
-    <KeyDescriptor use="encryption">
+    </KeyDescriptor></TMPL_LOOP>
+    <TMPL_LOOP name="encryption_keys"><KeyDescriptor use="encryption">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
-          <TMPL_VAR NAME="samlServicePublicKeyEnc">
+          <TMPL_VAR NAME="keyInfo">
       </ds:KeyInfo>
-    </KeyDescriptor>
+    </KeyDescriptor></TMPL_LOOP>
     <ArtifactResolutionService
       isDefault="<TMPL_VAR NAME="samlIDPSSODescriptorArtifactResolutionServiceArtifactDefault">"
       index="<TMPL_VAR NAME="samlIDPSSODescriptorArtifactResolutionServiceArtifactIndex">"
       Binding="<TMPL_VAR NAME="samlIDPSSODescriptorArtifactResolutionServiceArtifactBinding">"
       Location="<TMPL_VAR NAME="samlIDPSSODescriptorArtifactResolutionServiceArtifactLocation">" />
+    <TMPL_IF NAME="samlIDPSSODescriptorSingleLogoutServiceSOAPLocation">
     <SingleLogoutService
       Binding="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceSOAPBinding">"
       Location="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceSOAPLocation">" />
+    </TMPL_IF>
+    <TMPL_IF NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">
     <SingleLogoutService
       Binding="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPRedirectBinding">"
       Location="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPRedirectLocation">"
-      <TMPL_IF NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">
-      ResponseLocation="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">"
-      </TMPL_IF>/>
+      ResponseLocation="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">" />
+    </TMPL_IF>
+    <TMPL_IF NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">
     <SingleLogoutService
       Binding="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPPostBinding">"
       Location="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPPostLocation">"
-      <TMPL_IF NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">
-      ResponseLocation="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">"
-      </TMPL_IF>/>
+      ResponseLocation="<TMPL_VAR NAME="samlIDPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">" />
+    </TMPL_IF>
     <NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress</NameIDFormat>
     <NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:X509SubjectName</NameIDFormat>
     <NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:WindowsDomainQualifiedName</NameIDFormat>
@@ -284,36 +314,38 @@ __DATA__
       AuthnRequestsSigned="<TMPL_VAR NAME="samlSPSSODescriptorAuthnRequestsSigned">"
       WantAssertionsSigned="<TMPL_VAR NAME="samlSPSSODescriptorWantAssertionsSigned">"
       protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
-    <KeyDescriptor use="signing">
+    <TMPL_LOOP name="signing_keys"><KeyDescriptor use="signing">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
-          <TMPL_VAR NAME="samlServicePublicKeySig">
+          <TMPL_VAR NAME="keyInfo">
       </ds:KeyInfo>
-    </KeyDescriptor>
-    <KeyDescriptor use="encryption">
+    </KeyDescriptor></TMPL_LOOP>
+    <TMPL_LOOP name="encryption_keys"><KeyDescriptor use="encryption">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
-          <TMPL_VAR NAME="samlServicePublicKeyEnc">
+          <TMPL_VAR NAME="keyInfo">
       </ds:KeyInfo>
-    </KeyDescriptor>
+    </KeyDescriptor></TMPL_LOOP>
     <ArtifactResolutionService
       isDefault="<TMPL_VAR NAME="samlSPSSODescriptorArtifactResolutionServiceArtifactDefault">"
       index="<TMPL_VAR NAME="samlSPSSODescriptorArtifactResolutionServiceArtifactIndex">"
       Binding="<TMPL_VAR NAME="samlSPSSODescriptorArtifactResolutionServiceArtifactBinding">"
       Location="<TMPL_VAR NAME="samlSPSSODescriptorArtifactResolutionServiceArtifactLocation">" />
+    <TMPL_IF NAME="samlSPSSODescriptorSingleLogoutServiceSOAPLocation">
     <SingleLogoutService
       Binding="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceSOAPBinding">"
       Location="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceSOAPLocation">" />
+    </TMPL_IF>
+    <TMPL_IF NAME="samlSPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">
     <SingleLogoutService
       Binding="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPRedirectBinding">"
       Location="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPRedirectLocation">"
-      <TMPL_IF NAME="samlSPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">
-      ResponseLocation="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">"
-      </TMPL_IF>/>
+      ResponseLocation="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPRedirectResponseLocation">" />
+    </TMPL_IF>
+    <TMPL_IF NAME="samlSPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">
     <SingleLogoutService
       Binding="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPPostBinding">"
       Location="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPPostLocation">"
-      <TMPL_IF NAME="samlSPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">
-      ResponseLocation="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">"
-      </TMPL_IF>/>
+      ResponseLocation="<TMPL_VAR NAME="samlSPSSODescriptorSingleLogoutServiceHTTPPostResponseLocation">" />
+    </TMPL_IF>
     <NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:emailAddress</NameIDFormat>
     <NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:X509SubjectName</NameIDFormat>
     <NameIDFormat>urn:oasis:names:tc:SAML:1.1:nameid-format:WindowsDomainQualifiedName</NameIDFormat>
@@ -349,16 +381,16 @@ __DATA__
   <TMPL_UNLESS NAME="hideIDPMetadata">
   <AttributeAuthorityDescriptor
     protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
-    <KeyDescriptor use="signing">
+    <TMPL_LOOP name="signing_keys"><KeyDescriptor use="signing">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
-          <TMPL_VAR NAME="samlServicePublicKeySig">
+          <TMPL_VAR NAME="keyInfo">
       </ds:KeyInfo>
-    </KeyDescriptor>
-    <KeyDescriptor use="encryption">
+    </KeyDescriptor></TMPL_LOOP>
+    <TMPL_LOOP name="encryption_keys"><KeyDescriptor use="encryption">
       <ds:KeyInfo xmlns:ds="http://www.w3.org/2000/09/xmldsig#">
-          <TMPL_VAR NAME="samlServicePublicKeyEnc">
+          <TMPL_VAR NAME="keyInfo">
       </ds:KeyInfo>
-    </KeyDescriptor>
+    </KeyDescriptor></TMPL_LOOP>
     <AttributeService
       Binding="<TMPL_VAR NAME="samlAttributeAuthorityDescriptorAttributeServiceSOAPBinding">"
       Location="<TMPL_VAR NAME="samlAttributeAuthorityDescriptorAttributeServiceSOAPLocation">"/>
