@@ -2,7 +2,7 @@ package Lemonldap::NG::Common::Logger::_Duplicate;
 
 use strict;
 
-our $VERSION = '2.19.0';
+our $VERSION = '2.22.0';
 
 sub new {
     my $self = bless {}, shift;
@@ -14,6 +14,22 @@ sub new {
     return $self;
 }
 
+sub setRequestObj {
+    my $self = shift;
+    if ($self->{logger}->can('setRequestObj')) {
+        return $self->{logger}->setRequestObj(@_);
+    }
+    return;
+}
+
+sub clearRequestObj {
+    my $self = shift;
+    if ($self->{logger}->can('clearRequestObj')) {
+        return $self->{logger}->clearRequestObj(@_);
+    }
+    return;
+}
+
 sub AUTOLOAD {
     my $self = shift;
     no strict;
